@@ -1,58 +1,72 @@
 import base64
 import hashlib
+import os
+from http.cookiejar import LWPCookieJar
 
 import requests
 
 
 class irDataClient:
-    def __init__(self, username=None, password=None):
+    def __init__(self, username=None, password=None, cookie_file=None):
         self.authenticated = False
         self.session = requests.Session()
         self.base_url = "https://members-ng.iracing.com"
 
-        self._login(username=username, password=password)
+        self.username = username
+        self.encoded_password = self._encode_password(username, password)
 
-        self.cars = self.get_cars()
-        tracks = self.get_tracks()
-        track_assets = self.get_tracks_assets()
-        for track in tracks:
-            a = track_assets[str(track["track_id"])]
-            for key in a.keys():
-                track[key] = a[key]
+    def _encode_password(self, username, password):
+        initial_hash = hashlib.sha256((password + username.lower()).encode('utf-8')).digest()
 
-        self.tracks = tracks
+        return base64.b64encode(initial_hash).decode('utf-8')
 
-    def _login(self, username=None, password=None):
-        if not username or not password:
-            raise RuntimeError("Please supply a username and password")
+    def _login(self, cookie_file=None):
+        if cookie_file:
+            self.session.cookies = LWPCookieJar(cookie_file)
+            if not os.path.exists(cookie_file):
+                self.session.cookies.save()
+            else:
+                self.session.cookies.load(ignore_discard=True)
+        headers = {'Content-Type': 'application/json'}
+        data = {"email": self.username, "password": self.encoded_password}
 
-        # iRacing requires Base64 encoded string as of 2022 season 3
-        password_hash = hashlib.sha256(
-            (password + username.lower()).encode("utf-8")
-        ).digest()
-        password_b64 = base64.b64encode(password_hash).decode("utf-8")
-
-        payload = {"email": username, "password": password_b64}
-
-        r = self.session.post(self._build_url("/auth"), json=payload)
-
-        if r.status_code != 200:
-            raise RuntimeError("Error from iRacing: ", r.json())
-
-        self.authenticated = True
-        return True
+        try:
+            r = self.session.post('https://members-ng.iracing.com/auth', headers=headers, json=data, timeout=5.0)
+        except requests.Timeout:
+            raise RuntimeError("Login timed out")
+        except requests.ConnectionError:
+            raise RuntimeError("Connection error")
+        else:
+            response_data = r.json()
+            if r.status_code == 200 and response_data['authcode']:
+                if cookie_file:
+                    self.session.cookies.save(ignore_discard=True)
+                self.authenticated = True
+                return "Logged in"
+            else:
+                raise RuntimeError("Error from iRacing: ", response_data)
 
     def _build_url(self, endpoint):
         return self.base_url + endpoint
 
     def _get_resource_or_link(self, url, payload=None):
+        if not self.authenticated:
+            self._login()
+            return self._get_resource_or_link(url, payload=payload)
+
         r = self.session.get(url, params=payload)
+
+        if r.status_code == 401:
+            # unauthorised, likely due to a timeout, retry after a login
+            self.authenticated = False
+            
         if r.status_code != 200:
             raise RuntimeError(r.json())
-        if "link" in r.json().keys():
-            return [r.json()["link"], True]
+        data = r.json()
+        if not isinstance(data, list) and "link" in data.keys():
+            return [data["link"], True]
         else:
-            return [r.json(), False]
+            return [data, False]
 
     def _get_resource(self, endpoint, payload=None):
         request_url = self._build_url(endpoint)
@@ -72,6 +86,21 @@ class irDataClient:
 
         return output
 
+    @property
+    def cars(self):
+        return self.get_cars()
+
+    @property
+    def tracks(self):
+        tracks = self.get_tracks()
+        track_assets = self.get_tracks_assets()
+        for track in tracks:
+            a = track_assets[str(track["track_id"])]
+            for key in a.keys():
+                track[key] = a[key]
+
+        return tracks
+
     def constants_categories(self):
         return self._get_resource("/data/constants/categories")
 
@@ -272,8 +301,8 @@ class irDataClient:
         event_types=None,
         category_ids=None,
     ):
-        if not (season_year or season_quarter):
-            raise RuntimeError("Please supply Season Year and Season Quarter")
+        if not ((season_year and season_quarter) or (start_range_begin)):
+            raise RuntimeError("Please supply Season Year and Season Quarter or a date range")
 
         params = locals()
         payload = {}
@@ -306,7 +335,10 @@ class irDataClient:
             payload["cust_id"] = cust_id
 
         return self._get_resource("/data/member/chart_data", payload=payload)
-
+    
+    def member_info(self):
+        return self._get_resource("/data/member/info")
+    
     def stats_member_career(self, cust_id=None):
         if not cust_id:
             raise RuntimeError("Please supply a cust_id")
@@ -417,6 +449,10 @@ class irDataClient:
         payload = {"team_id": team_id, "include_licenses": include_licenses}
         return self._get_resource("/data/team/get", payload=payload)
 
+    def season_list(self, season_year, season_quarter):
+        payload = {"season_year": season_year, "season_quarter": season_quarter}
+        return self._get_resource("/data/season/list", payload=payload)
+    
     def series(self):
         return self._get_resource("/data/series/get")
 
